@@ -49,7 +49,7 @@ module Gitlab
         result = migrated_cache_fetch(cache.store, &block)
         return result unless specified.present?
 
-        cache.write(specified) if result != specified
+        cache.write(specified) if result.to_s != specified.to_s
         specified
       end
 
@@ -62,7 +62,9 @@ module Gitlab
       def migrated_cache_fetch(store, &block)
         migrations = context.signature[:migration_keys]&.map { |key| cache_key(key) } || []
         migrations.find do |old_key|
-          next unless (value = store.read(old_key))
+          value = store.read(old_key)
+
+          next unless value
 
           store.write(cache_key, value)
           store.delete(old_key)
