@@ -9,8 +9,8 @@ module Gitlab
         set_callback :run, :around, :manage_nested_stack
       end
 
-      def nest_experiment(other)
-        raise NestingError, "unable to nest the #{other.name} experiment within the #{name} experiment"
+      def nest_experiment(nested_experiment)
+        instance_exec(nested_experiment, &Configuration.nested_behavior)
       end
 
       private
@@ -25,15 +25,25 @@ module Gitlab
       class Stack
         include Singleton
 
-        @stack = []
+        delegate :pop, :length, :size, :[], to: :stack
 
         class << self
-          delegate :pop, :length, :size, :[], to: :@stack
+          delegate :pop, :push, :length, :size, :[], to: :instance
+        end
 
-          def push(instance)
-            @stack.last&.nest_experiment(instance)
-            @stack.push(instance)
-          end
+        def initialize
+          @thread_key = "#{self.class};#{object_id}".to_sym
+        end
+
+        def push(instance)
+          stack.last&.nest_experiment(instance)
+          stack.push(instance)
+        end
+
+        private
+
+        def stack
+          Thread.current[@thread_key] ||= []
         end
       end
     end
